<?php
// Based on https://www.drupal.org/project/facetapi_bonus/issues/1863780#comment-11206543

class FacetapiTokensHelper {

  public static function getFacetapiTokensData(&$pattern, &$types) {
    $searcher = variable_get('facetapi_tokens_search_api_searcher', '');

    if (!$searcher) {
      return;
    }

    // If we meet list<...[facetapi_active:XXX]...> string we replace it with
    // coma separated list of tokenized string.
    // So list<[facetapi_active:facet-label]: [facetapi_active:active-value]>
    // will be replaced with "type: node, author: admin" string if type and
    // author facets active.
    if ($adapter = facetapi_adapter_load($searcher)) {
      // Provide explicit information for $data for the key facetapi_results
      // replacements. This could be avoided if facetapi_tokens could provide
      // default explicit values based on facetapi_get_active_searchers.
      if (strpos($pattern, 'facetapi_results') !== FALSE) {
        $types['facetapi_results'] = array('facetapi_adapter' => $adapter);
      }

      // Provide explicit information for $data for the key facetapi_active_item
      // if any facetapi_active_item not wrapped in list<...> is found
      if (preg_match_all('/[^list<]\[facetapi_active:[^>]/', $pattern, $matches)>0) {
        // Select the item that was added last (so the most recently added filter).
        $items = $adapter->getAllActiveItems();
        if (!empty($items)) {
          $item = end($items);
          $item['adapter'] = $adapter;
          $types['facetapi_active_item'] = $item;
        }
      }

      // Provide explicit information for $data for the key facetapi_facet
      if (strpos($pattern, 'facetapi_facet') !== FALSE) {
        // Select the item that was added last (so the most recently added filter).
        $items = $adapter->getAllActiveItems();
        if (!empty($items)) {
          $item = end($items);
          $facet_name = $item['facets'][0];
          $types['facetapi_facet'] = facetapi_facet_load($facet_name, $searcher);
        }
      }

      // Handle the list iterator for the facetapi_active tokens.
      // Currently facetapi_tokens doesn't support the iterator on this token.
      if (strpos($pattern, 'list<') !== FALSE && strpos($pattern, 'facetapi_active') !== FALSE) {

        preg_match_all('/list<[^>]*\[facetapi_active:[^>]*>/', $pattern, $matches);
        $replacements = array();
        foreach ($matches[0] as $subpattern) {
          $string = substr($subpattern, 5, strlen($subpattern) - 6);
          // @see CurrentSearchItemActive::execute().
          $subreplacements = array();
          foreach ($adapter->getAllActiveItems() as $item) {
            // Adds adapter to the active item for token replacement.
            $item['adapter'] = $adapter;
            // Builds variables to pass to theme function.
            $data = array('facetapi_active_item' => $item);

            $subreplacements[] = token_replace($string, $data);
          }
          $replacements[$subpattern] = implode(', ', $subreplacements);
        }
        $pattern = str_replace(array_keys($replacements), array_values($replacements), $pattern);
      }
    }
  }
}
